#ifndef _PLK_SECURESCANSDK_API_H_
#define _PLK_SECURESCANSDK_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <sys/types.h>
#include "libusb-1.0/libusb.h"

//2020/10/22 Lupita add for parser MRZ Data
#define TD1_ROW_SIZE					30
#define TD3_ROW_SIZE					44

#define TYPE							"Type"
#define ANGLE						"Angle"

//MRZ
#define MRTDS						"MRTDs"
#define DOCUMENTNO					"DocumentNo"
#define FAMILYNAME					"Familyname"
#define GIVENNAME					"Givenname"
#define NATIONALITY					"Nationality"
#define BIRTHDAY						"Birthday"
#define PERSONALNO					"PersonalNo"
#define SEX							"Sex"
#define DATEOFEXPIRY				"Dateofexpiry"
#define EXPIRYDATE					"ExpiryDate"
#define ISSUESTATE					"IssueState"
#define NATIVENAME					"NativeName"

enum{
	VIDEO_FORMAT_MJPEG,
	VIDEO_FORMAT_YUYV
};

typedef struct tagControlSet
{	
	int (* pfn_wled_open) (libusb_device_handle *dev_handle);
	int (* pfn_wled_close) (libusb_device_handle *dev_handle);
	int (* pfn_ir_open) (libusb_device_handle *dev_handle);
	int (* pfn_ir_close) (libusb_device_handle *dev_handle);
	int (* pfn_uv_open) (libusb_device_handle *dev_handle);
	int (* pfn_uv_close) (libusb_device_handle *dev_handle);
	int (* pfn_buzzer_open) (libusb_device_handle *dev_handle);
	int (* pfn_buzzer_close) (libusb_device_handle *dev_handle);
	int (* pfn_sensor_enable) (libusb_device_handle *dev_handle);
	int (* pfn_sensor_disable) (libusb_device_handle *dev_handle);
	int (* pfn_get_sensor_status) (libusb_device_handle *dev_handle);
	int (* pfn_init) (libusb_device_handle *dev_handle);	
}SCANNER_CTL, *PSCANNER_CTL;

struct tagControlSet;
typedef struct tagSecureScan_Model
{
	char	*str_dev_name;
	char *str_vendor_name;
	int 	w_vendor;
	int 	w_product;
	PSCANNER_CTL pfn_controlset;
	int w_video_format;
	int w_img_width;
	int w_img_height;
	int w_fps;
}SECURESCAN_MODEL, *PSECURESCAN_MODEL;

typedef struct SensorDetectArg{
   void *handle;
   PSECURESCAN_MODEL scanner_model;
} sensor_detect_arg;

typedef struct SdkRevision{
	char drv_revision[32];
	char ip_revision[32];
} sdk_revision;


extern __attribute__((visibility("default"))) int plk_get_revision(sdk_revision* revision);
extern __attribute__((visibility("default"))) int plk_wled_open(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_wled_close(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_ir_open(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_ir_close(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_uv_open(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_uv_close(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_buzzer_open(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_buzzer_close(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_sensor_enable(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_get_sensor_status(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) void plk_attach_device(libusb_device_handle *dev_handle, int vid, int pid, PSECURESCAN_MODEL scanner_model);
extern __attribute__((visibility("default"))) int plk_init_scanner(libusb_device_handle *dev_handle, PSECURESCAN_MODEL scanner_model);
#ifdef __cplusplus
}
#endif

#endif
